<?php
/**
 * @author Acacia Dynamic
 * @copyright 2024 Acacia Dynamic
 * @license private
 */
require_once __DIR__ . '/../../../Classes/Wp_caller.php';
require_once __DIR__ . '/../../../Kernel/API/Entity/DemandeEntity.php';
require_once __DIR__ . '/../../../Kernel/API/Entity/GabaritEntity.php';
require_once __DIR__ . '/../../../Kernel/API/Linker/API_Linker.php';
require_once __DIR__ . '/../../../Kernel/API/Linker/CMS_Linker.php';
require_once __DIR__ . '/../../../Kernel/Translations/DigitalSignageTranslate.php';
require_once 'Manager.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class DemandeManager extends Manager
{
    private $managerEntityName = 'Demande';

    private $database_caller;

    public function __construct($caller)
    {
        if ($caller instanceof CMS_Linker) {
            $this->database_caller = $caller;
        }
    }

    /**
     * @throws DigitalSignageException
     */
    public static function set($cms_linker)
    {
        if (!$cms_linker instanceof CMS_Linker) {
            throw new DigitalSignageException(DigitalSignageTranslate::getTraduction('mauvais_cms_linker'));
        } else {
            return new DemandeManager($cms_linker);
        }
    }

    /**
     * @param array $needle
     *
     * @return array|DemandeEntity[]
     *
     * @throws DigitalSignageException
     */
    public function getDemandes($needle = [])
    {
        if (!empty($needle)) {
            $needle = '?' . implode('&', $needle);
        } else {
            $needle = '';
        }

        $demandes = API_Linker::get_entitys($this->managerEntityName . $needle, get_class(new DemandeEntity()), ForfaitManager::instance()->getToken());

        return $demandes;
    }

    /**
     * @param DemandeEntity $demande
     *
     * @throws DigitalSignageException
     */
    public function postDemande($demande)
    {
        $demandeIsGood = true;
        $datas = [];

        // Verification du type du parametre
        if (!$demande instanceof DemandeEntity) {
            throw new DigitalSignageException(DigitalSignageTranslate::getTraduction('attribut_faux', '1', 'postDemande', 'DemandeEntity'));
        }

        // Verification si tous les champs de base sont presents
        if (null === $demande->getGabarit()) {
            $demandeIsGood = false;
            $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_obligatoir', 'gabarit'));
        }

        if (null === $demande->getAutomatique()) {
            $demandeIsGood = false;
            $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_obligatoir', 'automatique'));
        }

        if (null === $demande->getProductId()) {
            $demandeIsGood = false;
            $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_obligatoir', 'product_id'));
        }

        // Récuperation du gabarit de la demande
        if ($demandeIsGood) {
            $gabarit = GabaritManager::instance()->getGabarit($demande->getGabarit());

            if (!$gabarit) {
                $demandeIsGood = false;
                $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_faux', 'gabarit'));
            }
        }

        // Vérification des champs du gabarit
        if ($demandeIsGood) {
            $demandeDatas = $demande->getData();

            foreach ($gabarit->getChamps() as $neededChamp) {
                if (!isset($demandeDatas[$neededChamp['name']])) {
                    $demandeIsGood = false;
                    $this->addUserMessage(DigitalSignageTranslate::getTraduction('champ_obligatoir', $neededChamp['name']));
                } else {
                    // Si le parametre est une image alors on fait un check supplementaire
                    if ($neededChamp['type'] == 'picture') {
                        // Verification de l'existance du fichier
                        if (!file_exists($demandeDatas[$neededChamp['name']])) {
                            $demandeIsGood = false; // echo 'image_introuvable . ';var_dump($neededChamp);die;
                            $this->addUserMessage(DigitalSignageTranslate::getTraduction('image_introuvable', $demandeDatas[$neededChamp['name']]));
                        } else {
                            // Vérification de l'exctension du fichier
                            $extension = explode('.', $demandeDatas[$neededChamp['name']]);
                            $extension = array_pop($extension);
                            if (!in_array($extension, ['png', 'jpg', 'jpeg', 'gif', 'webp'])) {
                                $demandeIsGood = false;
                                $this->addUserMessage(DigitalSignageTranslate::getTraduction('image_type_faux', $demandeDatas[$neededChamp['name']]));
                            } else {
                                if (function_exists('curl_file_create')) { // php 5.5+
                                    $cFile = curl_file_create($demandeDatas[$neededChamp['name']]);
                                } else {
                                    $cFile = '@' . realpath($demandeDatas[$neededChamp['name']]);
                                }

                                $datas[$neededChamp['name']] = $cFile;
                            }
                        }
                    } else {
                        $datas[$neededChamp['name']] = $demandeDatas[$neededChamp['name']];
                    }
                }
            }
        }

        // Verification des champs obligatoirs des gaabrit
        foreach (['video_name', 'video_comment'] as $neededChamp) {
            $datas[$neededChamp] = (isset($demandeDatas[$neededChamp])) ? $demandeDatas[$neededChamp] : '';
        }

        // Envois de la demandes

        if ($demandeIsGood) {
            $datas = array_merge($datas, [
                'gabarit' => $demande->getGabarit(),
                'automatique' => $demande->getAutomatique(),
                'product_id' => $demande->getProductId(),
                'video_name' => $demande->getVideoName(),
                'video_comment' => $demande->getVideoComment(),
                'downloaded' => '0',
                'video_deleted' => '0',
                'youtube_data' => $demande->getYoutubeData(),
            ]);

            $demande = API_Linker::post($this->managerEntityName, $datas, ForfaitManager::instance()->getToken());

            if ($demande->getResponseCode() >= 400) {
                foreach ($demande->getErrors() as $error) {
                    $this->addUserMessage(DigitalSignageTranslate::getTraduction('api_erreur', $error));
                }
            } else {
                $demandeE = new DemandeEntity();
                $demandeDatas = $demande->getDatas();
                $demandeE->hydrate($demandeDatas[0]);

                return $demandeE;
            }
        }

        return false;
    }

    /**
     * @param $sql
     *
     * @return array|DemandeEntity[]|mixed
     */
    public function getLocalDemandes($sql)
    {
        $videos = $this->database_caller->execute_video_request($sql);

        if ($videos) {
            foreach ($videos as $key => $video) {
                $videoEntity = new DemandeEntity();
                $video = $videoEntity->hydrate($video);

                if ($video) {
                    $gabarit = GabaritManager::instance()->getGabarit($video->getGabarit());
                    $video->setGabarit($gabarit);
                    $videos[$key] = $video;
                }
            }

            return $videos;
        }

        return [];
    }
}
